// Copyright (c) 2015 Vadim Macagon
// MIT License, see LICENSE file for full terms.
var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
/// <reference path="../typings/lib/tsd.d.ts" />
var child_process_1 = require('child_process');
var readline = require('readline');
var os = require('os');
var path = require('path');
var events = require('events');
var parser = require('./mi_output_parser');
var mi_output_1 = require('./mi_output');
var pty = require('pty.js');
var DebugCommand = (function () {
    /**
     * @param cmd MI command string (minus the token and dash prefix).
     * @param token Token that can be used to match up the command with a response.
     * @param done Callback to invoke once a response is received for the command.
     */
    function DebugCommand(cmd, token, done) {
        this.token = token;
        this.text = cmd;
        this.done = done;
    }
    return DebugCommand;
})();
/**
 * Used to indicate failure of a MI command sent to the debugger.
 */
var CommandFailedError = (function () {
    function CommandFailedError(message, command, code, token) {
        this.name = "CommandFailedError";
        this.message = message;
        this.code = code;
        this.command = command;
        this.token = token;
    }
    return CommandFailedError;
})();
exports.CommandFailedError = CommandFailedError;
/**
 * Used to indicate the response to an MI command didn't match the expected format.
 */
var MalformedResponseError = (function () {
    /**
     * @param message The description of the error.
     * @param response The malformed response text (usually just the relevant part).
     * @param command The command text that was sent to the debugger (minus token and dash prefix).
     * @param token Token of the command (if the command had one).
     */
    function MalformedResponseError(message, response, command, token) {
        this.message = message;
        this.response = response;
        this.command = command;
        this.token = token;
        this.name = "MalformedResponseError";
    }
    return MalformedResponseError;
})();
exports.MalformedResponseError = MalformedResponseError;
(function (TargetStopReason) {
    /** A breakpoint was hit. */
    TargetStopReason[TargetStopReason["BreakpointHit"] = 0] = "BreakpointHit";
    /** A step instruction finished. */
    TargetStopReason[TargetStopReason["EndSteppingRange"] = 1] = "EndSteppingRange";
    /** A step-out instruction finished. */
    TargetStopReason[TargetStopReason["FunctionFinished"] = 2] = "FunctionFinished";
    /** The target finished executing and terminated normally. */
    TargetStopReason[TargetStopReason["ExitedNormally"] = 3] = "ExitedNormally";
    /** The target was signalled. */
    TargetStopReason[TargetStopReason["SignalReceived"] = 4] = "SignalReceived";
    /** The target encountered an exception (this is LLDB specific). */
    TargetStopReason[TargetStopReason["ExceptionReceived"] = 5] = "ExceptionReceived";
    /** Catch-all for any of the other numerous reasons. */
    TargetStopReason[TargetStopReason["Unrecognized"] = 6] = "Unrecognized";
})(exports.TargetStopReason || (exports.TargetStopReason = {}));
var TargetStopReason = exports.TargetStopReason;
/** Indicates how much information should be retrieved when calling
 *  [[DebugSession.getLocalVariables]].
 */
(function (VariableDetailLevel) {
    /** Only variable names will be retrieved, not their types or values. */
    VariableDetailLevel[VariableDetailLevel["None"] = 0] = "None";
    /** Only variable names and values will be retrieved, not their types. */
    VariableDetailLevel[VariableDetailLevel["All"] = 1] = "All";
    /**
     * The name and type will be retrieved for all variables, however values will only be retrieved
     * for simple variable types (not arrays, structures or unions).
     */
    VariableDetailLevel[VariableDetailLevel["Simple"] = 2] = "Simple";
})(exports.VariableDetailLevel || (exports.VariableDetailLevel = {}));
var VariableDetailLevel = exports.VariableDetailLevel;
/** Output format specifiers for watch values. */
(function (WatchFormatSpec) {
    WatchFormatSpec[WatchFormatSpec["Binary"] = 0] = "Binary";
    WatchFormatSpec[WatchFormatSpec["Decimal"] = 1] = "Decimal";
    WatchFormatSpec[WatchFormatSpec["Hexadecimal"] = 2] = "Hexadecimal";
    WatchFormatSpec[WatchFormatSpec["Octal"] = 3] = "Octal";
    /**
     * This specifier is used to indicate that one of the other ones should be automatically chosen
     * based on the expression type, for example `Decimal` for integers, `Hexadecimal` for pointers.
     */
    WatchFormatSpec[WatchFormatSpec["Default"] = 4] = "Default";
})(exports.WatchFormatSpec || (exports.WatchFormatSpec = {}));
var WatchFormatSpec = exports.WatchFormatSpec;
/** A watch may have one or more of these attributes associated with it. */
(function (WatchAttribute) {
    /** Indicates the watch value can be modified. */
    WatchAttribute[WatchAttribute["Editable"] = 0] = "Editable";
    /**
     * Indicates the watch value can't be modified. This will be the case for any watch with
     * children (at least when implemented correctly by the debugger, *cough* not LLDB-MI *cough*).
     */
    WatchAttribute[WatchAttribute["NonEditable"] = 1] = "NonEditable";
})(exports.WatchAttribute || (exports.WatchAttribute = {}));
var WatchAttribute = exports.WatchAttribute;
/** Output format specifiers for register values. */
(function (RegisterValueFormatSpec) {
    RegisterValueFormatSpec[RegisterValueFormatSpec["Binary"] = 0] = "Binary";
    RegisterValueFormatSpec[RegisterValueFormatSpec["Decimal"] = 1] = "Decimal";
    RegisterValueFormatSpec[RegisterValueFormatSpec["Hexadecimal"] = 2] = "Hexadecimal";
    RegisterValueFormatSpec[RegisterValueFormatSpec["Octal"] = 3] = "Octal";
    RegisterValueFormatSpec[RegisterValueFormatSpec["Raw"] = 4] = "Raw";
    /**
     * This specifier is used to indicate that one of the other ones should be automatically chosen.
     */
    RegisterValueFormatSpec[RegisterValueFormatSpec["Default"] = 5] = "Default";
})(exports.RegisterValueFormatSpec || (exports.RegisterValueFormatSpec = {}));
var RegisterValueFormatSpec = exports.RegisterValueFormatSpec;
/**
 * A debug session provides two-way communication with a debugger process via the GDB/LLDB
 * machine interface.
 *
 * Currently commands are queued and executed one at a time in the order they are issued,
 * a command will not be executed until all the previous commands have been acknowledged by the
 * debugger.
 *
 * Out of band notifications from the debugger are emitted via events, the names of these events
 * are provided by the EVENT_XXX static constants.
 */
var DebugSession = (function (_super) {
    __extends(DebugSession, _super);
    /**
     * In most cases [[startDebugSession]] should be used to construct new instances.
     *
     * @param inStream Debugger responses and notifications will be read from this stream.
     * @param outStream Debugger commands will be written to this stream.
     */
    function DebugSession(inStream, outStream) {
        _super.call(this);
        this.outStream = outStream;
        this.lineReader = readline.createInterface({
            input: inStream,
            output: null
        });
        this.lineReader.on('line', this.parseDebbugerOutput.bind(this));
        this.nextCmdId = 1;
        this.cmdQueue = [];
        this.cleanupWasCalled = false;
    }
    Object.defineProperty(DebugSession.prototype, "logger", {
        get: function () {
            return this._logger;
        },
        set: function (logger) {
            this._logger = logger;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Ends the debugging session.
     *
     * @param notifyDebugger If **false** the session is cleaned up immediately without waiting for
     *                       the debugger to respond (useful in cases where the debugger terminates
     *                       unexpectedly). If **true** the debugger is asked to exit, and once the
     *                       request is acknowldeged the session is cleaned up.
     */
    DebugSession.prototype.end = function (notifyDebugger) {
        var _this = this;
        if (notifyDebugger === void 0) { notifyDebugger = true; }
        return new Promise(function (resolve, reject) {
            var cleanup = function (err, data) {
                _this.cleanupWasCalled = true;
                _this.lineReader.close();
                err ? reject(err) : resolve();
            };
            if (!_this.cleanupWasCalled) {
                notifyDebugger ? _this.enqueueCommand(new DebugCommand('gdb-exit', null, cleanup))
                    : cleanup(null, null);
            }
            ;
        });
    };
    /**
     * Returns `true` if [[EVENT_FUNCTION_FINISHED]] can be emitted during this debugging session.
     *
     * LLDB-MI currently doesn't emit [[EVENT_FUNCTION_FINISHED]] after stepping out of a function,
     * instead it emits [[EVENT_STEP_FINISHED]] just like it does for any other stepping operation.
     */
    DebugSession.prototype.canEmitFunctionFinishedNotification = function () {
        return false;
    };
    DebugSession.prototype.emitExecNotification = function (name, data) {
        switch (name) {
            case 'running':
                this.emit(DebugSession.EVENT_TARGET_RUNNING, data['thread-id']);
                break;
            case 'stopped':
                if (this.logger) {
                    this.logger.debug(data);
                }
                var standardNotify = {
                    reason: parseTargetStopReason(data.reason),
                    threadId: parseInt(data['thread-id'], 10),
                    stoppedThreads: parseStoppedThreadsList(data['stopped-threads']),
                    processCore: data.core
                };
                this.emit(DebugSession.EVENT_TARGET_STOPPED, standardNotify);
                // emit a more specialized event for notifications that contain additional info
                switch (standardNotify.reason) {
                    case TargetStopReason.BreakpointHit:
                        var breakpointNotify = {
                            reason: standardNotify.reason,
                            threadId: standardNotify.threadId,
                            stoppedThreads: standardNotify.stoppedThreads,
                            processorCore: standardNotify.processorCore,
                            breakpointId: parseInt(data.bkptno, 10),
                            frame: extractFrameInfo(data.frame)
                        };
                        this.emit(DebugSession.EVENT_BREAKPOINT_HIT, breakpointNotify);
                        break;
                    case TargetStopReason.EndSteppingRange:
                        var stepNotify = {
                            reason: standardNotify.reason,
                            threadId: standardNotify.threadId,
                            stoppedThreads: standardNotify.stoppedThreads,
                            processorCore: standardNotify.processorCore,
                            frame: extractFrameInfo(data.frame)
                        };
                        this.emit(DebugSession.EVENT_STEP_FINISHED, stepNotify);
                        break;
                    case TargetStopReason.FunctionFinished:
                        var stepOutNotify = {
                            reason: standardNotify.reason,
                            threadId: standardNotify.threadId,
                            stoppedThreads: standardNotify.stoppedThreads,
                            processorCore: standardNotify.processorCore,
                            frame: extractFrameInfo(data.frame),
                            resultVar: data['gdb-result-var'],
                            returnValue: data['return-value']
                        };
                        this.emit(DebugSession.EVENT_FUNCTION_FINISHED, stepOutNotify);
                        break;
                    case TargetStopReason.SignalReceived:
                        var signalNotify = {
                            reason: standardNotify.reason,
                            threadId: standardNotify.threadId,
                            stoppedThreads: standardNotify.stoppedThreads,
                            processorCore: standardNotify.processorCore,
                            signalCode: data.signal,
                            signalName: data['signal-name'],
                            signalMeaning: data['signal-meaning']
                        };
                        this.emit(DebugSession.EVENT_SIGNAL_RECEIVED, signalNotify);
                        break;
                    case TargetStopReason.ExceptionReceived:
                        var exceptionNotify = {
                            reason: standardNotify.reason,
                            threadId: standardNotify.threadId,
                            stoppedThreads: standardNotify.stoppedThreads,
                            processorCore: standardNotify.processorCore,
                            exception: data.exception
                        };
                        this.emit(DebugSession.EVENT_EXCEPTION_RECEIVED, exceptionNotify);
                        break;
                }
                break;
            default:
                // TODO: log and keep on going
                break;
        }
    };
    DebugSession.prototype.emitAsyncNotification = function (name, data) {
        var shlibInfo;
        switch (name) {
            case 'thread-group-added':
                this.emit(DebugSession.EVENT_THREAD_GROUP_ADDED, data);
                break;
            case 'thread-group-removed':
                this.emit(DebugSession.EVENT_THREAD_GROUP_REMOVED, data);
                break;
            case 'thread-group-started':
                this.emit(DebugSession.EVENT_THREAD_GROUP_STARTED, data);
                break;
            case 'thread-group-exited':
                this.emit(DebugSession.EVENT_THREAD_GROUP_EXITED, { id: data.id, exitCode: data['exit-code'] });
                break;
            case 'thread-created':
                this.emit(DebugSession.EVENT_THREAD_CREATED, { id: data.id, groupId: data['group-id'] });
                break;
            case 'thread-exited':
                this.emit(DebugSession.EVENT_THREAD_EXITED, { id: data.id, groupId: data['group-id'] });
                break;
            case 'thread-selected':
                this.emit(DebugSession.EVENT_THREAD_SELECTED, data);
                break;
            case 'library-loaded':
                this.emit(DebugSession.EVENT_LIB_LOADED, {
                    id: data.id,
                    targetName: data['target-name'],
                    hostName: data['host-name'],
                    threadGroup: data['thread-group'],
                    symbolsPath: data['symbols-path'],
                    loadAddress: data.loaded_addr
                });
                break;
            case 'library-unloaded':
                this.emit(DebugSession.EVENT_LIB_UNLOADED, {
                    id: data.id,
                    targetName: data['target-name'],
                    hostName: data['host-name'],
                    threadGroup: data['thread-group'],
                    symbolsPath: data['symbols-path'],
                    loadAddress: data.loaded_addr
                });
                break;
            default:
                // TODO: log and keep on going
                break;
        }
        ;
    };
    /**
     * Parse a single line containing a response to a MI command or some sort of async notification.
     */
    DebugSession.prototype.parseDebbugerOutput = function (line) {
        // '(gdb)' (or '(gdb) ' in some cases) is used to indicate the end of a set of output lines 
        // from the debugger, but since we process each line individually as it comes in this 
        // particular marker is of no use
        if (line.match(/^\(gdb\)\s*/) || (line === '')) {
            return;
        }
        var cmdQueuePopped = false;
        try {
            var result = parser.parse(line);
        }
        catch (err) {
            if (this.logger) {
                this.logger.error(err, 'Attempted to parse: ->' + line + '<-');
            }
            throw err;
        }
        switch (result.recordType) {
            case mi_output_1.RecordType.Done:
            case mi_output_1.RecordType.Running:
            case mi_output_1.RecordType.Connected:
            case mi_output_1.RecordType.Exit:
            case mi_output_1.RecordType.Error:
                // this record is a response for the last command that was sent to the debugger, 
                // which is the command at the front of the queue
                var cmd = this.cmdQueue.shift();
                cmdQueuePopped = true;
                // todo: check that the token in the response matches the one sent with the command
                if (cmd.done) {
                    if (result.recordType === mi_output_1.RecordType.Error) {
                        cmd.done(new CommandFailedError(result.data.msg, cmd.text, result.data.code, cmd.token), null);
                    }
                    else {
                        cmd.done(null, result.data);
                    }
                }
                break;
            case mi_output_1.RecordType.AsyncExec:
                this.emitExecNotification(result.data[0], result.data[1]);
                break;
            case mi_output_1.RecordType.AsyncNotify:
                this.emitAsyncNotification(result.data[0], result.data[1]);
                break;
            case mi_output_1.RecordType.DebuggerConsoleOutput:
                this.emit(DebugSession.EVENT_DBG_CONSOLE_OUTPUT, result.data);
                break;
            case mi_output_1.RecordType.TargetOutput:
                this.emit(DebugSession.EVENT_TARGET_OUTPUT, result.data);
                break;
            case mi_output_1.RecordType.DebuggerLogOutput:
                this.emit(DebugSession.EVENT_DBG_LOG_OUTPUT, result.data);
                break;
        }
        // if a command was popped from the qeueu we can send through the next command
        if (cmdQueuePopped && (this.cmdQueue.length > 0)) {
            this.sendCommandToDebugger(this.cmdQueue[0]);
        }
    };
    /**
     * Sends an MI command to the debugger process.
     */
    DebugSession.prototype.sendCommandToDebugger = function (command) {
        var cmdStr;
        if (command.token) {
            cmdStr = command.token + "-" + command.text;
        }
        else {
            cmdStr = '-' + command.text;
        }
        if (this.logger) {
            this.logger.info(cmdStr);
        }
        this.outStream.write(cmdStr + '\n');
    };
    /**
     * Adds an MI command to the back of the command queue.
     *
     * If the command queue is empty when this method is called then the command is dispatched
     * immediately, otherwise it will be dispatched after all the previously queued commands are
     * processed.
     */
    DebugSession.prototype.enqueueCommand = function (command) {
        this.cmdQueue.push(command);
        if (this.cmdQueue.length === 1) {
            this.sendCommandToDebugger(this.cmdQueue[0]);
        }
    };
    /**
     * Sends an MI command to the debugger.
     *
     * @param command Full MI command string, excluding the optional token and dash prefix.
     * @param token Token to be prefixed to the command string (must consist only of digits).
     * @returns A promise that will be resolved when the command response is received.
     */
    DebugSession.prototype.executeCommand = function (command, token) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.enqueueCommand(new DebugCommand(command, token, function (err, data) { err ? reject(err) : resolve(); }));
        });
    };
    /**
     * Sends an MI command to the debugger and returns the response.
     *
     * @param command Full MI command string, excluding the optional token and dash prefix.
     * @param token Token to be prefixed to the command string (must consist only of digits).
     * @param transformOutput This function will be invoked with the output of the MI Output parser
     *                        and should transform that output into an instance of type `T`.
     * @returns A promise that will be resolved when the command response is received.
     */
    DebugSession.prototype.getCommandOutput = function (command, token, transformOutput) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.enqueueCommand(new DebugCommand(command, token, function (err, data) {
                if (err) {
                    reject(err);
                }
                else {
                    try {
                        resolve(transformOutput ? transformOutput(data) : data);
                    }
                    catch (err) {
                        reject(err);
                    }
                }
            }));
        });
    };
    /**
     * Sets the executable file to be debugged, the symbol table will also be read from this file.
     *
     * This must be called prior to [[connectToRemoteTarget]] when setting up a remote debugging
     * session.
     *
     * @param file This would normally be a full path to the host's copy of the executable to be
     *             debugged.
     */
    DebugSession.prototype.setExecutableFile = function (file) {
        // NOTE: While the GDB/MI spec. contains multiple -file-XXX commands that allow the
        // executable and symbol files to be specified separately the LLDB MI driver
        // currently (30-Mar-2015) only supports this one command.
        return this.executeCommand("file-exec-and-symbols " + file);
    };
    /**
     * Sets the terminal to be used by the next inferior that's launched.
     *
     * @param slaveName Name of the slave end of a pseudoterminal that should be associated with
     *                  the inferior, see `man pty` for an overview of pseudoterminals.
     */
    DebugSession.prototype.setInferiorTerminal = function (slaveName) {
        return this.executeCommand('inferior-tty-set ' + slaveName);
    };
    /**
     * Connects the debugger to a remote target.
     *
     * @param host
     * @param port
     */
    DebugSession.prototype.connectToRemoteTarget = function (host, port) {
        return this.executeCommand("target-select remote " + host + ":" + port);
    };
    //
    // Breakpoint Commands
    //
    /**
     * Adds a new breakpoint.
     *
     * @param location The location at which a breakpoint should be added, can be specified in the
     *                 following formats:
     *                 - function_name
     *                 - filename:line_number
     *                 - filename:function_name
     *                 - address
     * @param options.isTemp Set to **true** to create a temporary breakpoint which will be
     *                       automatically removed after being hit.
     * @param options.isHardware Set to **true** to create a hardware breakpoint
     *                           (presently not supported by LLDB MI).
     * @param options.isPending Set to **true** if the breakpoint should still be created even if
     *                          the location cannot be parsed (e.g. it refers to uknown files or
     *                          functions).
     * @param options.isDisabled Set to **true** to create a breakpoint that is initially disabled,
     *                           otherwise the breakpoint will be enabled by default.
     * @param options.isTracepoint Set to **true** to create a tracepoint
     *                             (presently not supported by LLDB MI).
     * @param options.condition The debugger will only stop the program execution when this
     *                          breakpoint is hit if the condition evaluates to **true**.
     * @param options.ignoreCount The number of times the breakpoint should be hit before it takes
     *                            effect, zero (the default) means the breakpoint will stop the
     *                            program every time it's hit.
     * @param options.threadId Restricts the new breakpoint to the given thread.
     */
    DebugSession.prototype.addBreakpoint = function (location, options) {
        var cmd = 'break-insert';
        if (options) {
            if (options.isTemp) {
                cmd = cmd + ' -t';
            }
            if (options.isHardware) {
                cmd = cmd + ' -h';
            }
            if (options.isPending) {
                cmd = cmd + ' -f';
            }
            if (options.isDisabled) {
                cmd = cmd + ' -d';
            }
            if (options.isTracepoint) {
                cmd = cmd + ' -a';
            }
            if (options.condition) {
                cmd = cmd + ' -c ' + options.condition;
            }
            if (options.ignoreCount !== undefined) {
                cmd = cmd + ' -i ' + options.ignoreCount;
            }
            if (options.threadId !== undefined) {
                cmd = cmd + ' -p ' + options.threadId;
            }
        }
        return this.getCommandOutput(cmd + ' ' + location, null, function (output) {
            return extractBreakpointInfo(output);
        });
    };
    /**
     * Removes a breakpoint.
     */
    DebugSession.prototype.removeBreakpoint = function (breakId) {
        return this.executeCommand('break-delete ' + breakId);
    };
    /**
     * Removes multiple breakpoints.
     */
    DebugSession.prototype.removeBreakpoints = function (breakIds) {
        // FIXME: LLDB MI driver only supports removing one breakpoint at a time,
        //        so multiple breakpoints need to be removed one by one.
        return this.executeCommand('break-delete ' + breakIds.join(' '));
    };
    /**
     * Enables a breakpoint.
     */
    DebugSession.prototype.enableBreakpoint = function (breakId) {
        return this.executeCommand('break-enable ' + breakId);
    };
    /**
     * Disables a breakpoint.
     */
    DebugSession.prototype.disableBreakpoint = function (breakId) {
        return this.executeCommand('break-disable ' + breakId);
    };
    /**
     * Tells the debugger to ignore a breakpoint the next `ignoreCount` times it's hit.
     *
     * @param breakId Identifier of the breakpoint for which the ignore count should be set.
     * @param ignoreCount The number of times the breakpoint should be hit before it takes effect,
     *                    zero means the breakpoint will stop the program every time it's hit.
     */
    DebugSession.prototype.ignoreBreakpoint = function (breakId, ignoreCount) {
        return this.getCommandOutput("break-after " + breakId + " " + ignoreCount, null, function (output) { return extractBreakpointInfo(output); });
    };
    /**
     * Sets the condition under which a breakpoint should take effect when hit.
     *
     * @param breakId Identifier of the breakpoint for which the condition should be set.
     * @param condition Expression to evaluate when the breakpoint is hit, if it evaluates to
     *                  **true** the breakpoint will stop the program, otherwise the breakpoint
     *                  will have no effect.
     */
    DebugSession.prototype.setBreakpointCondition = function (breakId, condition) {
        return this.executeCommand("break-condition " + breakId + " " + condition);
    };
    //
    // Program Execution Commands
    //
    /**
     * Sets the commandline arguments to be passed to the target process next time it is started
     * using [[startTarget]].
     */
    DebugSession.prototype.setTargetArguments = function (args) {
        return this.executeCommand('exec-arguments ' + args);
    };
    /**
     * Executes an inferior from the beginning until it exits.
     *
     * Execution may stop before the inferior finishes running due to a number of reasons,
     * for example a breakpoint being hit.
     * [[EVENT_TARGET_STOPPED]] will be emitted when execution stops.
     *
     * @param options.threadGroup *(GDB specific)* The identifier of the thread group to start,
     *                            if omitted the currently selected inferior will be started.
     * @param options.stopAtStart *(GDB specific)* If `true` then execution will stop at the start
     *                            of the main function.
     */
    DebugSession.prototype.startInferior = function (options) {
        var fullCmd = 'exec-run';
        if (options) {
            if (options.threadGroup) {
                fullCmd = fullCmd + ' --thread-group ' + options.threadGroup;
            }
            if (options.stopAtStart) {
                fullCmd = fullCmd + ' --start';
            }
        }
        return this.executeCommand(fullCmd, null);
    };
    /**
     * Executes all inferiors from the beginning until they exit.
     *
     * Execution may stop before an inferior finishes running due to a number of reasons,
     * for example a breakpoint being hit.
     * [[EVENT_TARGET_STOPPED]] will be emitted when execution stops.
     *
     * @param stopAtStart *(GDB specific)* If `true` then execution will stop at the start
     *                    of the main function.
     */
    DebugSession.prototype.startAllInferiors = function (stopAtStart) {
        var fullCmd = 'exec-run --all';
        if (stopAtStart) {
            fullCmd = fullCmd + ' --start';
        }
        return this.executeCommand(fullCmd, null);
    };
    /**
     * Kills the currently selected inferior.
     */
    DebugSession.prototype.abortInferior = function () {
        return this.executeCommand('exec-abort');
    };
    /**
     * Resumes execution of an inferior, execution may stop at any time due to a number of reasons,
     * for example a breakpoint being hit.
     * [[EVENT_TARGET_STOPPED]] will be emitted when execution stops.
     *
     * @param options.threadGroup *(GDB specific)* Identifier of the thread group to resume,
     *                            if omitted the currently selected inferior is resumed.
     * @param options.reverse *(GDB specific)* If **true** the inferior is executed in reverse.
     */
    DebugSession.prototype.resumeInferior = function (options) {
        var fullCmd = 'exec-continue';
        if (options) {
            if (options.threadGroup) {
                fullCmd = fullCmd + ' --thread-group ' + options.threadGroup;
            }
            if (options.reverse) {
                fullCmd = fullCmd + ' --reverse';
            }
        }
        return this.executeCommand(fullCmd, null);
    };
    /**
     * Resumes execution of all inferiors.
     *
     * @param reverse *(GDB specific)* If `true` the inferiors are executed in reverse.
     */
    DebugSession.prototype.resumeAllInferiors = function (reverse) {
        var fullCmd = 'exec-continue --all';
        if (reverse) {
            fullCmd = fullCmd + ' --reverse';
        }
        return this.executeCommand(fullCmd, null);
    };
    /**
     * Interrupts execution of an inferior.
     * [[EVENT_TARGET_STOPPED]] will be emitted when execution stops.
     *
     * @param options.threadGroup The identifier of the thread group to interrupt, if omitted the
     *                            currently selected inferior will be interrupted.
     */
    DebugSession.prototype.interruptInferior = function (threadGroup) {
        var fullCmd = 'exec-interrupt';
        if (threadGroup) {
            fullCmd = fullCmd + ' --thread-group ' + threadGroup;
        }
        return this.executeCommand(fullCmd, null);
    };
    /**
     * Interrupts execution of all threads in all inferiors.
     *
     * [[EVENT_TARGET_STOPPED]] will be emitted when execution stops.
     */
    DebugSession.prototype.interruptAllInferiors = function () {
        return this.executeCommand('exec-interrupt --all', null);
    };
    /**
     * Resumes execution of the target until the beginning of the next source line is reached.
     * If a function is called while the target is running then execution stops on the first
     * source line of the called function.
     * [[EVENT_TARGET_STOPPED]] will be emitted when execution stops.
     *
     * @param options.threadId Identifier of the thread to execute the command on.
     * @param options.reverse *(GDB specific)* If **true** the target is executed in reverse.
     */
    DebugSession.prototype.stepIntoLine = function (options) {
        return this.executeCommand(appendExecCmdOptions('exec-step', options));
    };
    /**
     * Resumes execution of the target until the beginning of the next source line is reached.
     * [[EVENT_TARGET_STOPPED]] will be emitted when execution stops.
     *
     * @param options.threadId Identifier of the thread to execute the command on.
     * @param options.reverse *(GDB specific)* If **true** the target is executed in reverse until
     *                        the beginning of the previous source line is reached.
     */
    DebugSession.prototype.stepOverLine = function (options) {
        return this.executeCommand(appendExecCmdOptions('exec-next', options));
    };
    /**
     * Executes one instruction, if the instruction is a function call then execution stops at the
     * beginning of the function.
     * [[EVENT_TARGET_STOPPED]] will be emitted when execution stops.
     *
     * @param options.threadId Identifier of the thread to execute the command on.
     * @param options.reverse *(GDB specific)* If **true** the target is executed in reverse until
     *                        the previous instruction is reached.
     */
    DebugSession.prototype.stepIntoInstruction = function (options) {
        return this.executeCommand(appendExecCmdOptions('exec-step-instruction', options));
    };
    /**
     * Executes one instruction, if the instruction is a function call then execution continues
     * until the function returns.
     * [[EVENT_TARGET_STOPPED]] will be emitted when execution stops.
     *
     * @param options.threadId Identifier of the thread to execute the command on.
     * @param options.reverse *(GDB specific)* If **true** the target is executed in reverse until
     *                        the previous instruction is reached.
     */
    DebugSession.prototype.stepOverInstruction = function (options) {
        return this.executeCommand(appendExecCmdOptions('exec-next-instruction', options));
    };
    /**
     * Resumes execution of the target until the current function returns.
     * [[EVENT_TARGET_STOPPED]] will be emitted when execution stops.
     *
     * @param options.threadId Identifier of the thread to execute the command on.
     * @param options.reverse *(GDB specific)* If **true** the target is executed in reverse.
     */
    DebugSession.prototype.stepOut = function (options) {
        return this.executeCommand(appendExecCmdOptions('exec-finish', options));
    };
    //
    // Stack Inspection Commands
    //
    /**
     * Retrieves information about a stack frame.
     *
     * @param options.threadId The thread for which the stack depth should be retrieved,
     *                         defaults to the currently selected thread if not specified.
     * @param options.frameLevel Stack index of the frame for which to retrieve locals,
     *                           zero for the innermost frame, one for the frame from which the call
     *                           to the innermost frame originated, etc. Defaults to the currently
     *                           selected frame if not specified. If a value is provided for this
     *                           option then `threadId` must be specified as well.
     */
    DebugSession.prototype.getStackFrame = function (options) {
        var fullCmd = 'stack-info-frame';
        if (options) {
            if (options.threadId !== undefined) {
                fullCmd = fullCmd + ' --thread ' + options.threadId;
            }
            if (options.frameLevel !== undefined) {
                fullCmd = fullCmd + ' --frame ' + options.frameLevel;
            }
        }
        return this.getCommandOutput(fullCmd, null, function (output) {
            return extractStackFrameInfo(output.frame);
        });
    };
    /**
     * Retrieves the current depth of the stack.
     *
     * @param options.threadId The thread for which the stack depth should be retrieved,
     *                         defaults to the currently selected thread if not specified.
     * @param options.maxDepth *(GDB specific)* If specified the returned stack depth will not exceed
     *                         this number.
     */
    DebugSession.prototype.getStackDepth = function (options) {
        var fullCmd = 'stack-info-depth';
        if (options) {
            if (options.threadId !== undefined) {
                fullCmd = fullCmd + ' --thread ' + options.threadId;
            }
            if (options.maxDepth !== undefined) {
                fullCmd = fullCmd + ' ' + options.maxDepth;
            }
        }
        return this.getCommandOutput(fullCmd, null, function (output) {
            return parseInt(output.depth);
        });
    };
    /**
     * Retrieves the frames currently on the stack.
     *
     * The `lowFrame` and `highFrame` options can be used to limit the number of frames retrieved,
     * if both are supplied only the frame with levels in that range (inclusive) are retrieved.
     * If either `lowFrame` or `highFrame` option is omitted (but not both) then only a single
     * frame corresponding to that level is retrieved.
     *
     * @param options.threadId The thread for which the stack frames should be retrieved,
     *                         defaults to the currently selected thread if not specified.
     * @param options.noFrameFilters *(GDB specific)* If `true` the Python frame filters will not be
     *                               executed.
     * @param options.lowFrame Must not be larger than the actual number of frames on the stack.
     * @param options.highFrame May be larger than the actual number of frames on the stack, in which
     *                          case only the existing frames will be retrieved.
     */
    DebugSession.prototype.getStackFrames = function (options) {
        var fullCmd = 'stack-list-frames';
        if (options) {
            if (options.threadId !== undefined) {
                fullCmd = fullCmd + ' --thread' + options.threadId;
            }
            if (options.noFrameFilters === true) {
                fullCmd = fullCmd + ' --no-frame-filters';
            }
            if ((options.lowFrame !== undefined) && (options.highFrame !== undefined)) {
                fullCmd = fullCmd + (" " + options.lowFrame + " " + options.highFrame);
            }
            else if (options.lowFrame !== undefined) {
                fullCmd = fullCmd + (" " + options.lowFrame + " " + options.lowFrame);
            }
            else if (options.highFrame !== undefined) {
                fullCmd = fullCmd + (" " + options.highFrame + " " + options.highFrame);
            }
        }
        return this.getCommandOutput(fullCmd, null, function (output) {
            var data = output.stack.frame;
            if (Array.isArray(data)) {
                return data.map(function (frame) { return extractStackFrameInfo(frame); });
            }
            else {
                return [extractStackFrameInfo(data)];
            }
        });
    };
    /**
     * Retrieves a list of all the arguments for the specified frames.
     *
     * The `lowFrame` and `highFrame` options can be used to limit the frames for which arguments
     * are retrieved. If both are supplied only the frames with levels in that range (inclusive) are
     * taken into account, if both are omitted the arguments of all frames currently on the stack
     * will be retrieved.
     *
     * Note that while it's possible to specify a frame range of one frame in order to retrieve the
     * arguments of a single frame it's better to just use [[getStackFrameVariables]] instead.
     *
     * @param detail Specifies what information should be retrieved for each argument.
     * @param options.threadId The thread for which arguments should be retrieved,
     *                         defaults to the currently selected thread if not specified.
     * @param options.noFrameFilters *(GDB specific)* If `true` then Python frame filters will not be
     *                               executed.
     * @param options.skipUnavailable If `true` information about arguments that are not available
     *                                will not be retrieved.
     * @param options.lowFrame Must not be larger than the actual number of frames on the stack.
     * @param options.highFrame May be larger than the actual number of frames on the stack, in which
     *                          case only the existing frames will be retrieved.
     */
    DebugSession.prototype.getStackFrameArgs = function (detail, options) {
        var fullCmd = 'stack-list-arguments';
        if (options) {
            if (options.threadId !== undefined) {
                fullCmd = fullCmd + ' --thread ' + options.threadId;
            }
            if (options.noFrameFilters === true) {
                fullCmd = fullCmd + ' --no-frame-filters';
            }
            if (options.skipUnavailable === true) {
                fullCmd = fullCmd + ' --skip-unavailable';
            }
        }
        fullCmd = fullCmd + ' ' + detail;
        if (options) {
            if ((options.lowFrame !== undefined) && (options.highFrame !== undefined)) {
                fullCmd = fullCmd + (" " + options.lowFrame + " " + options.highFrame);
            }
            else if ((options.lowFrame !== undefined) && (options.highFrame === undefined)) {
                throw new Error("highFrame option must be provided to getStackFrameArgs() if lowFrame option is used.");
            }
            else if ((options.lowFrame === undefined) && (options.highFrame !== undefined)) {
                throw new Error("lowFrame option must be provided to getStackFrameArgs() if highFrame option is used.");
            }
        }
        return this.getCommandOutput(fullCmd, null, function (output) {
            var data = output['stack-args'];
            if (Array.isArray(data.frame)) {
                // data is in the form: { frame: [{ level: 0, args: [...] }, { level: 1, args: arg1 }, ...]
                return data.frame.map(function (frame) {
                    return {
                        level: parseInt(frame.level, 10),
                        args: Array.isArray(frame.args) ? frame.args : [frame.args]
                    };
                });
            }
            else {
                // data is in the form: { frame: { level: 0, args: [...] }
                return [{
                        level: parseInt(data.frame.level, 10),
                        args: Array.isArray(data.frame.args) ? data.frame.args : [data.frame.args]
                    }];
            }
        });
    };
    /**
     * Retrieves a list of all arguments and local variables in the specified frame.
     *
     * @param detail Specifies what information to retrieve for each argument or local variable.
     * @param options.threadId The thread for which variables should be retrieved,
     *                         defaults to the currently selected thread if not specified.
     * @param options.frameLevel Stack index of the frame for which to retrieve locals,
     *                           zero for the innermost frame, one for the frame from which the call
     *                           to the innermost frame originated, etc. Defaults to the currently
     *                           selected frame if not specified.
     * @param options.noFrameFilters *(GDB specific)* If `true` then Python frame filters will not be
     *                               executed.
     * @param options.skipUnavailable If `true` information about variables that are not available
     *                                will not be retrieved.
     */
    DebugSession.prototype.getStackFrameVariables = function (detail, options) {
        var fullCmd = 'stack-list-variables';
        if (options) {
            if (options.threadId !== undefined) {
                fullCmd = fullCmd + ' --thread ' + options.threadId;
            }
            if (options.frameLevel !== undefined) {
                fullCmd = fullCmd + ' --frame ' + options.frameLevel;
            }
            if (options.noFrameFilters === true) {
                fullCmd = fullCmd + ' --no-frame-filters';
            }
            if (options.skipUnavailable === true) {
                fullCmd = fullCmd + ' --skip-unavailable';
            }
        }
        fullCmd = fullCmd + ' ' + detail;
        return this.getCommandOutput(fullCmd, null, function (output) {
            var args = [];
            var locals = [];
            output.variables.forEach(function (varInfo) {
                if (varInfo.arg === '1') {
                    args.push({ name: varInfo.name, value: varInfo.value, type: varInfo.type });
                }
                else {
                    locals.push({ name: varInfo.name, value: varInfo.value, type: varInfo.type });
                }
            });
            return { args: args, locals: locals };
        });
    };
    //
    // Watch Manipulation (aka Variable Objects)
    //
    /**
     * Creates a new watch to monitor the value of the given expression.
     *
     * @param expression Any expression valid in the current language set (so long as it doesn't
     *                   begin with a `*`), or one of the following:
     *                   - a memory cell address, e.g. `*0x0000000000400cd0`
     *                   - a CPU register name, e.g. `$sp`
     * @param options.id Unique identifier for the new watch, if omitted one is auto-generated.
     *                   Auto-generated identifiers begin with the letters `var` and are followed by
     *                   one or more digits, when providing your own identifiers it's best to use a
     *                   different naming scheme that doesn't clash with auto-generated identifiers.
     * @param options.threadId The thread within which the watch expression will be evaluated.
     *                         *Default*: the currently selected thread.
     * @param options.threadGroup
     * @param options.frameLevel The index of the stack frame within which the watch expression will
     *                           be evaluated initially, zero for the innermost stack frame. Note that
     *                           if `frameLevel` is specified then `threadId` must also be specified.
     *                           *Default*: the currently selected frame.
     * @param options.frameAddress *(GDB specific)* Address of the frame within which the expression
     *                             should be evaluated.
     * @param options.isFloating Set to `true` if the expression should be re-evaluated every time
     *                           within the current frame, i.e. it's not bound to a specific frame.
     *                           Set to `false` if the expression should be bound to the frame within
     *                           which the watch is created.
     *                           *Default*: `false`.
     */
    DebugSession.prototype.addWatch = function (expression, options) {
        var fullCmd = 'var-create';
        var id = '-'; // auto-generate id
        var addr = '*'; // use current frame
        if (options) {
            if (options.id) {
                id = options.id;
            }
            if (options.threadId !== undefined) {
                fullCmd = fullCmd + ' --thread ' + options.threadId;
            }
            if (options.threadGroup) {
                fullCmd = fullCmd + ' --thread-group ' + options.threadGroup;
            }
            if (options.frameLevel !== undefined) {
                fullCmd = fullCmd + ' --frame ' + options.frameLevel;
            }
            if (options.isFloating === true) {
                addr = '@';
            }
            else if (options.frameAddress) {
                addr = options.frameAddress;
            }
        }
        fullCmd = fullCmd + (" " + id + " " + addr + " " + expression);
        return this.getCommandOutput(fullCmd, null, function (output) {
            return {
                id: output.name,
                childCount: parseInt(output.numchild),
                value: output.value,
                expressionType: output['type'],
                threadId: parseInt(output['thread-id']),
                hasMoreChildren: output.has_more !== '0',
                isDynamic: output.dynamic === '1',
                displayHint: output.displayhint
            };
        });
    };
    /**
     * Destroys a previously created watch.
     *
     * @param id Identifier of the watch to destroy.
     */
    DebugSession.prototype.removeWatch = function (id) {
        return this.executeCommand('var-delete ' + id);
    };
    /**
     * Updates the state of an existing watch.
     *
     * @param id Identifier of the watch to update.
     */
    DebugSession.prototype.updateWatch = function (id, detail) {
        var fullCmd = 'var-update';
        if (detail !== undefined) {
            fullCmd = fullCmd + ' ' + detail;
        }
        fullCmd = fullCmd + ' ' + id;
        return this.getCommandOutput(fullCmd, null, function (output) {
            return output.changelist.map(function (data) {
                return {
                    id: data.name,
                    childCount: (data.new_num_children ? parseInt(data.new_num_children) : undefined),
                    value: data.value,
                    expressionType: data.new_type,
                    isInScope: data.in_scope === 'true',
                    isObsolete: data.in_scope === 'invalid',
                    hasTypeChanged: data.type_changed === 'true',
                    isDynamic: data.dynamic === '1',
                    displayHint: data.displayhint,
                    hasMoreChildren: data.has_more === '1',
                    newChildren: data.new_children
                };
            });
        });
    };
    /**
     * Retrieves a list of direct children of the specified watch.
     *
     * A watch is automatically created for each child that is retrieved (if one doesn't already exist).
     * The `from` and `to` options can be used to retrieve a subset of children starting from child
     * index `from` and up to (but excluding) child index `to`, note that this currently doesn't work
     * on LLDB.
     *
     * @param id Identifier of the watch whose children should be retrieved.
     * @param options.detail One of:
     *     - [[VariableDetailLevel.None]]: Do not retrieve values of children, this is the default.
     *     - [[VariableDetailLevel.All]]: Retrieve values for all children.
     *     - [[VariableDetailLevel.Simple]]: Only retrieve values of children that have a simple type.
     * @param options.from Zero-based index of the first child to retrieve, if less than zero the
     *                     range is reset. `to` must also be set in order for this option to have any
     *                     effect.
     * @param options.to Zero-based index +1 of the last child to retrieve, if less than zero the
     *                   range is reset. `from` must also be set in order for this option to have any
     *                   effect.
     */
    DebugSession.prototype.getWatchChildren = function (id, options) {
        var fullCmd = 'var-list-children';
        if (options && (options.detail !== undefined)) {
            fullCmd = fullCmd + ' ' + options.detail;
        }
        fullCmd = fullCmd + ' ' + id;
        if (options && (options.from !== undefined) && (options.to !== undefined)) {
            fullCmd = fullCmd + ' ' + options.from + ' ' + options.to;
        }
        return this.getCommandOutput(fullCmd, null, function (output) {
            return extractWatchChildren(output.children);
        });
    };
    /**
     * Sets the output format for the value of a watch.
     *
     * @param id Identifier of the watch for which the format specifier should be set.
     * @param formatSpec The output format for the watch value.
     * @returns A promise that will be resolved with the value of the watch formatted using the
     *          provided `formatSpec`.
     */
    DebugSession.prototype.setWatchValueFormat = function (id, formatSpec) {
        var fullCmd = ("var-set-format " + id + " ") + watchFormatSpecToStringMap.get(formatSpec);
        return this.getCommandOutput(fullCmd, null, function (output) {
            if (output.value) {
                return output.value; // GDB-MI
            }
            else {
                return output.changelist[0].value; // LLDB-MI
            }
        });
    };
    /**
     * Evaluates the watch expression and returns the result.
     *
     * @param id Identifier of the watch whose value should be retrieved.
     * @param formatSpec The output format for the watch value.
     * @returns A promise that will be resolved with the value of the watch.
     */
    DebugSession.prototype.getWatchValue = function (id, formatSpec) {
        var fullCmd = 'var-evaluate-expression';
        if (formatSpec !== undefined) {
            fullCmd = fullCmd + ' -f ' + watchFormatSpecToStringMap.get(formatSpec);
        }
        fullCmd = fullCmd + ' ' + id;
        return this.getCommandOutput(fullCmd, null, function (output) {
            return output.value;
        });
    };
    /**
     * Sets the value of the watch expression to the value of the given expression.
     *
     * @param id Identifier of the watch whose value should be modified.
     * @param expression The value of this expression will be assigned to the watch expression.
     * @returns A promise that will be resolved with the new value of the watch.
     */
    DebugSession.prototype.setWatchValue = function (id, expression) {
        return this.getCommandOutput("var-assign " + id + " \"" + expression + "\"", null, function (output) {
            return output.value;
        });
    };
    /**
     * Retrives a list of attributes for the given watch.
     *
     * @param id Identifier of the watch whose attributes should be retrieved.
     * @returns A promise that will be resolved with the list of watch attributes.
     */
    DebugSession.prototype.getWatchAttributes = function (id) {
        var cmd = 'var-show-attributes ' + id;
        return this.getCommandOutput(cmd, null, function (output) {
            if (output.status) {
                return [stringToWatchAttributeMap.get(output.status)];
            }
            else if (output.attr) {
                if (Array.isArray(output.attr)) {
                    return output.attr.map(function (attr) {
                        return stringToWatchAttributeMap.get(attr);
                    });
                }
                else {
                    return [stringToWatchAttributeMap.get(output.attr)];
                }
            }
            throw new MalformedResponseError('Expected to find "status" or "attr", found neither.', output, cmd);
        });
    };
    /**
     * Retrieves an expression that can be evaluated in the current context to obtain the watch value.
     *
     * @param id Identifier of the watch whose path expression should be retrieved.
     * @returns A promise that will be resolved with the path expression of the watch.
     */
    DebugSession.prototype.getWatchExpression = function (id) {
        var cmd = 'var-info-path-expression ' + id;
        return this.getCommandOutput(cmd, null, function (output) {
            if (output.path_expr) {
                return output.path_expr;
            }
            throw new MalformedResponseError('Expected to find "path_expr".', output, cmd);
        });
    };
    //
    // Data Inspection & Manipulation
    //
    /**
     * Evaluates the given expression within the target process and returns the result.
     *
     * The expression may contain function calls, which will be executed synchronously.
     *
     * @param expression The expression to evaluate.
     * @param options.threadId The thread within which the expression should be evaluated.
     *                         *Default*: the currently selected thread.
     * @param options.frameLevel The index of the stack frame within which the expression should
     *                           be evaluated, zero for the innermost stack frame. Note that
     *                           if `frameLevel` is specified then `threadId` must also be specified.
     *                           *Default*: the currently selected frame.
     * @returns A promise that will be resolved with the value of the expression.
     */
    DebugSession.prototype.evaluateExpression = function (expression, options) {
        var fullCmd = 'data-evaluate-expression';
        if (options) {
            if (options.threadId !== undefined) {
                fullCmd = fullCmd + ' --thread ' + options.threadId;
            }
            if (options.frameLevel !== undefined) {
                fullCmd = fullCmd + ' --frame ' + options.frameLevel;
            }
        }
        fullCmd = fullCmd + (" \"" + expression + "\"");
        return this.getCommandOutput(fullCmd, null, function (output) {
            if (output.value) {
                return output.value;
            }
            throw new MalformedResponseError('Expected to find "value".', output, fullCmd);
        });
    };
    /**
     * Attempts to read all accessible memory regions in the given range.
     *
     * @param address Start of the range from which memory should be read, this can be a literal
     *                address (e.g. `0x00007fffffffed30`) or an expression (e.g. `&someBuffer`) that
     *                evaluates to the desired address.
     * @param numBytesToRead Number of bytes that should be read.
     * @param options.byteOffset Offset in bytes relative to `address` from which to begin reading.
     * @returns A promise that will be resolved with a list of memory blocks that were read.
     */
    DebugSession.prototype.readMemory = function (address, numBytesToRead, options) {
        var fullCmd = 'data-read-memory-bytes';
        if (options && options.byteOffset) {
            fullCmd = fullCmd + ' -o ' + options.byteOffset;
        }
        fullCmd = fullCmd + (" \"" + address + "\" " + numBytesToRead);
        return this.getCommandOutput(fullCmd, null, function (output) {
            if (output.memory) {
                return output.memory;
            }
            throw new MalformedResponseError('Expected to find "memory".', output, fullCmd);
        });
    };
    /**
     * Retrieves a list of register names for the current target.
     *
     * @param registers List of numbers corresponding to the register names to be retrieved.
     *                  If this argument is omitted all register names will be retrieved.
     * @returns A promise that will be resolved with a list of register names.
     */
    DebugSession.prototype.getRegisterNames = function (registers) {
        var fullCmd = 'data-list-register-names';
        if (registers && (registers.length > 0)) {
            fullCmd = fullCmd + ' ' + registers.join(' ');
        }
        return this.getCommandOutput(fullCmd, null, function (output) {
            if (output['register-names']) {
                return output['register-names'];
            }
            throw new MalformedResponseError('Expected to find "register-names".', output, fullCmd);
        });
    };
    /**
     * Retrieves the values of registers.
     *
     * @param formatSpec Specifies how the register values should be formatted.
     * @param options.registers Register numbers of the registers for which values should be retrieved.
     *                          If this option is omitted the values of all registers will be retrieved.
     * @param options.skipUnavailable *(GDB specific)* If `true` only values of available registers
     *                                will be retrieved.
     * @param options.threadId Identifier of the thread from which register values should be retrieved.
     *                         If this option is omitted it will default to the currently selected thread.
     *                         NOTE: This option is not currently supported by LLDB-MI.
     * @param options.frameLevel Index of the frame from which register values should be retrieved.
     *                           This is a zero-based index, zero corresponds to the innermost frame
     *                           on the stack. If this option is omitted it will default to the
     *                           currently selected frame.
     *                           NOTE: This option is not currently supported by LLDB-MI.
     * @returns A promise that will be resolved with a map of register numbers to register values.
     */
    DebugSession.prototype.getRegisterValues = function (formatSpec, options) {
        var fullCmd = 'data-list-register-values';
        if (options) {
            if (options.threadId !== undefined) {
                fullCmd = fullCmd + ' --thread ' + options.threadId;
            }
            if (options.frameLevel !== undefined) {
                fullCmd = fullCmd + ' --frame ' + options.frameLevel;
            }
            if (options.skipUnavailable) {
                fullCmd = fullCmd + ' --skip-unavailable';
            }
        }
        fullCmd = fullCmd + ' ' + registerValueFormatSpecToCodeMap.get(formatSpec);
        if (options && options.registers && (options.registers.length > 0)) {
            fullCmd = fullCmd + ' ' + options.registers.join(' ');
        }
        return this.getCommandOutput(fullCmd, null, function (output) {
            var registers = output['register-values'];
            var registerMap = new Map();
            if (registers) {
                registers.forEach(function (register) {
                    registerMap.set(parseInt(register.number), register.value);
                });
                return registerMap;
            }
            throw new MalformedResponseError('Expected to find "register-values".', output, fullCmd);
        });
    };
    /**
     * Retrieves assembly instructions within the specified address range.
     *
     * No source line information will be provided for the assembly instructions that are retrieved,
     * if such information is required [[disassembleAddressRangeByLine]] should be used instead.
     *
     * @param start Start of the address range to disassemble. GDB allows this to be an expression
     *              that can be evaluated to obtain the address (e.g. $pc), however LLDB-MI only
     *              accepts number literals (e.g. 0x4009cc).
     * @param end End of the address range to disassemble, same caveats apply as for `start`.
     * @param showOpcodes If `true` the raw opcode bytes will be retrieved for each instruction.
     * @returns A promise that will be resolved with a list of assembly instructions (and associated
     *          meta-data).
     */
    DebugSession.prototype.disassembleAddressRange = function (start, end, showOpcodes) {
        var fullCmd = ("data-disassemble -s " + start + " -e " + end + " -- ") + (showOpcodes ? '2' : '0');
        return this.getCommandOutput(fullCmd, null, function (output) {
            if (output.asm_insns) {
                return extractAsmInstructions(output.asm_insns);
            }
            throw new MalformedResponseError('Expected to find "asm_insns".', output, fullCmd);
        });
    };
    /**
     * Retrieves assembly instructions within a specified address range grouped by source line.
     *
     * If source line information is not required [[disassembleAddressRange]] should be used instead.
     *
     * @param start Start of the address range to disassemble. GDB allows this to be an expression
     *              that can be evaluated to obtain the address (e.g. $pc), however LLDB-MI only
     *              accepts number literals (e.g. 0x4009cc).
     * @param end End of the address range to disassemble, same caveats apply as for `start`.
     * @param showOpcodes If `true` the raw opcode bytes will be retrieved for each instruction.
     * @returns A promise that will be resolved with a list lines, each of which will contain one
     *          or more assembly instructions (and associated meta-data).
     */
    DebugSession.prototype.disassembleAddressRangeByLine = function (start, end, showOpcodes) {
        var fullCmd = ("data-disassemble -s " + start + " -e " + end + " -- ") + (showOpcodes ? '3' : '1');
        return this.getCommandOutput(fullCmd, null, function (output) {
            if (output.asm_insns) {
                return extractAsmBySourceLine(output.asm_insns);
            }
            throw new MalformedResponseError('Expected to find "asm_insns".', output, fullCmd);
        });
    };
    /**
     * Retrieves assembly instructions for the specified source file.
     *
     * No source line information will be provided for the assembly instructions that are retrieved,
     * if such information is required [[disassembleFileByLine]] should be used instead.
     *
     * @param filename Source file to disassemble, e.g. main.cpp
     * @param line Line number in `filename` to disassemble around.
     * @param options.maxInstructions Maximum number of assembly instructions to retrieve.
     *                                If this option is ommitted the entire function at the specified
     *                                source line will be disassembled.
     * @param options.showOpcodes If `true` the raw opcode bytes will be retrieved for each instruction.
     * @returns A promise that will be resolved with a list of assembly instructions (and associated
     *          meta-data).
     */
    DebugSession.prototype.disassembleFile = function (filename, line, options) {
        var fullCmd = "data-disassemble -f " + filename + " -l " + line;
        if (options && (options.maxInstructions !== undefined)) {
            fullCmd = fullCmd + ' -n ' + options.maxInstructions;
        }
        fullCmd = fullCmd + ' -- ' + ((options && options.showOpcodes) ? '2' : '0');
        return this.getCommandOutput(fullCmd, null, function (output) {
            if (output.asm_insns) {
                return extractAsmInstructions(output.asm_insns);
            }
            throw new MalformedResponseError('Expected to find "asm_insns".', output, fullCmd);
        });
    };
    /**
     * Retrieves assembly instructions for the specified source file grouped by source line.
     *
     * If source line information is not required [[disassembleFile]] should be used instead.
     *
     * @param filename Source file to disassemble, e.g. main.cpp
     * @param line Line number in `filename` to disassemble around.
     * @param options.maxInstructions Maximum number of assembly instructions to retrieve.
     *                                If this option is ommitted the entire function at the specified
     *                                source line will be disassembled.
     * @param options.showOpcodes If `true` the raw opcode bytes will be retrieved for each instruction.
     * @returns A promise that will be resolved with a list lines, each of which will contain one
     *          or more assembly instructions (and associated meta-data).
     */
    DebugSession.prototype.disassembleFileByLine = function (filename, line, options) {
        var fullCmd = "data-disassemble -f " + filename + " -l " + line;
        if (options && (options.maxInstructions !== undefined)) {
            fullCmd = fullCmd + ' -n ' + options.maxInstructions;
        }
        fullCmd = fullCmd + ' -- ' + ((options && options.showOpcodes) ? '3' : '1');
        return this.getCommandOutput(fullCmd, null, function (output) {
            if (output.asm_insns) {
                return extractAsmBySourceLine(output.asm_insns);
            }
            throw new MalformedResponseError('Expected to find "asm_insns".', output, fullCmd);
        });
    };
    /**
     * Emitted when a thread group is added by the debugger, it's possible the thread group
     * hasn't yet been associated with a running program.
     *
     * Listener function should have the signature:
     * ~~~
     * (notification: [[ThreadGroupAddedNotify]]) => void
     * ~~~
     * @event
     */
    DebugSession.EVENT_THREAD_GROUP_ADDED = 'thdgrpa';
    /**
     * Emitted when a thread group is removed by the debugger.
     *
     * Listener function should have the signature:
     * ~~~
     * (notification: [[ThreadGroupRemovedNotify]]) => void
     * ~~~
     * @event
     */
    DebugSession.EVENT_THREAD_GROUP_REMOVED = 'thdgrpr';
    /**
     * Emitted when a thread group is associated with a running program,
     * either because the program was started or the debugger was attached to it.
     *
     * Listener function should have the signature:
     * ~~~
     * (notification: [[ThreadGroupStartedNotify]]) => void
     * ~~~
     * @event
     */
    DebugSession.EVENT_THREAD_GROUP_STARTED = 'thdgrps';
    /**
     * Emitted when a thread group ceases to be associated with a running program,
     * either because the program terminated or the debugger was dettached from it.
     *
     * Listener function should have the signature:
     * ~~~
     * (notification: [[ThreadGroupExitedNotify]]) => void
     * ~~~
     * @event
     */
    DebugSession.EVENT_THREAD_GROUP_EXITED = 'thdgrpe';
    /**
     * Emitted when a thread is created.
     *
     * Listener function should have the signature:
     * ~~~
     * (notification: [[ThreadCreatedNotify]]) => void
     * ~~~
     * @event
     */
    DebugSession.EVENT_THREAD_CREATED = 'thdc';
    /**
     * Emitted when a thread exits.
     *
     * Listener function should have the signature:
     * ~~~
     * (notification: [[ThreadExitedNotify]]) => void
     * ~~~
     * @event
     */
    DebugSession.EVENT_THREAD_EXITED = 'thde';
    /**
     * Emitted when the debugger changes the current thread selection.
     *
     * Listener function should have the signature:
     * ~~~
     * (notification: [[ThreadSelectedNotify]]) => void
     * ~~~
     * @event
     */
    DebugSession.EVENT_THREAD_SELECTED = 'thds';
    /**
     * Emitted when a new library is loaded by the program being debugged.
     *
     * Listener function should have the signature:
     * ~~~
     * (notification: [[LibLoadedNotify]]) => void
     * ~~~
     * @event
     */
    DebugSession.EVENT_LIB_LOADED = 'libload';
    /**
     * Emitted when a library is unloaded by the program being debugged.
     *
     * Listener function should have the signature:
     * ~~~
     * (notification: [[LibUnloadedNotify]]) => void
     * ~~~
     * @event
     */
    DebugSession.EVENT_LIB_UNLOADED = 'libunload';
    /**
     * Emitted when some console output from the debugger becomes available,
     * usually in response to a CLI command.
     *
     * Listener function should have the signature:
     * ~~~
     * (output: string) => void
     * ~~~
     * @event
     */
    DebugSession.EVENT_DBG_CONSOLE_OUTPUT = 'conout';
    /**
     * Emitted when some console output from the target becomes available.
     *
     * Listener function should have the signature:
     * ~~~
     * (output: string) => void
     * ~~~
     * @event
     */
    DebugSession.EVENT_TARGET_OUTPUT = 'targetout';
    /**
     * Emitted when log output from the debugger becomes available.
     *
     * Listener function should have the signature:
     * ~~~
     * (output: string) => void
     * ~~~
     * @event
     */
    DebugSession.EVENT_DBG_LOG_OUTPUT = 'dbgout';
    /**
     * Emitted when the target starts running.
     *
     * The `threadId` passed to the listener indicates which specific thread is now running,
     * a value of **"all"** indicates all threads are running. According to the GDB/MI spec.
     * no interaction with a running thread is possible after this notification is produced until
     * it is stopped again.
     *
     * Listener function should have the signature:
     * ~~~
     * (threadId: string) => void
     * ~~~
     * @event
     */
    DebugSession.EVENT_TARGET_RUNNING = 'targetrun';
    /**
     * Emitted when the target stops running.
     *
     * Listener function should have the signature:
     * ~~~
     * (notification: [[TargetStoppedNotify]]) => void
     * ~~~
     * @event
     */
    DebugSession.EVENT_TARGET_STOPPED = 'targetstop';
    /**
     * Emitted when the target stops running because a breakpoint was hit.
     *
     * Listener function should have the signature:
     * ~~~
     * (notification: [[BreakpointHitNotify]]) => void
     * ~~~
     * @event
     */
    DebugSession.EVENT_BREAKPOINT_HIT = 'brkpthit';
    /**
     * Emitted when the target stops due to a stepping operation finishing.
     *
     * Listener function should have the signature:
     * ~~~
     * (notification: [[StepFinishedNotify]]) => void
     * ~~~
     * @event
     */
    DebugSession.EVENT_STEP_FINISHED = 'endstep';
    /**
     * Emitted when the target stops due to a step-out operation finishing.
     *
     * NOTE: Currently this event will not be emitted by LLDB-MI, it will only be emitted by GDB-MI,
     * so for the time being use [[EVENT_STEP_FINISHED]] with LLDB-MI.
     *
     * Listener function should have the signature:
     * ~~~
     * (notification: [[StepOutFinishedNotify]]) => void
     * ~~~
     * @event
     */
    DebugSession.EVENT_FUNCTION_FINISHED = 'endfunc';
    /**
     * Emitted when the target stops running because it received a signal.
     *
     * Listener function should have the signature:
     * ~~~
     * (notification: [[SignalReceivedNotify]]) => void
     * ~~~
     * @event
     */
    DebugSession.EVENT_SIGNAL_RECEIVED = 'signal';
    /**
     * Emitted when the target stops running due to an exception.
     *
     * Listener function should have the signature:
     * ~~~
     * (notification: [[ExceptionReceivedNotify]]) => void
     * ~~~
     * @event
     */
    DebugSession.EVENT_EXCEPTION_RECEIVED = 'exception';
    return DebugSession;
})(events.EventEmitter);
exports.DebugSession = DebugSession;
/**
 * Appends some common options used by -exec-* MI commands to the given string.
 *
 * @returns The result of appending the options to the input string.
 */
function appendExecCmdOptions(input, options) {
    var cmd = input;
    if (options) {
        if (options.threadId !== undefined) {
            cmd = cmd + ' --thread ' + options.threadId;
        }
        if (options.reverse) {
            cmd = cmd + ' --reverse';
        }
    }
    return cmd;
}
/**
 * Converts the output produced by the MI Output parser from the response to the
 * -break-insert and -break-after MI commands into a more useful form.
 */
function extractBreakpointInfo(data) {
    return {
        id: data.bkpt['number'],
        breakpointType: data.bkpt['type'],
        catchpointType: data.bkpt['catch-type'],
        isTemp: (data.bkpt.disp !== undefined) ? (data.bkpt.disp === 'del') : undefined,
        isEnabled: (data.bkpt.enabled !== undefined) ? (data.bkpt.enabled === 'y') : undefined,
        address: data.bkpt.addr,
        func: data.bkpt.func,
        filename: data.bkpt.file || data.bkpt.filename,
        fullname: data.bkpt.fullname,
        line: data.bkpt.line,
        at: data.bkpt.at,
        pending: data.bkpt.pending,
        evaluatedBy: data.bkpt['evaluated-by'],
        threadId: data.bkpt.thread,
        condition: data.bkpt.cond,
        ignoreCount: data.bkpt.ignore,
        enableCount: data.bkpt.enable,
        mask: data.bkpt.mask,
        passCount: data.bkpt.pass,
        originalLocation: data.bkpt['original-location'],
        hitCount: data.bkpt.times,
        isInstalled: (data.bkpt.installed !== undefined) ? (data.bkpt.installed === 'y') : undefined,
        what: data.bkpt.what
    };
}
/**
 * Creates an object that conforms to the IFrameInfo interface from the output of the
 * MI Output parser.
 */
function extractFrameInfo(data) {
    return {
        func: data.func,
        args: data.args,
        address: data.addr,
        filename: data.file,
        fullname: data.fullname,
        line: data.line ? parseInt(data.line, 10) : undefined,
    };
}
/**
 * Creates an object that conforms to the IStackFrameInfo interface from the output of the
 * MI Output parser.
 */
function extractStackFrameInfo(data) {
    return {
        level: parseInt(data.level),
        func: data.func,
        address: data.addr,
        filename: data.file,
        fullname: data.fullname,
        line: data.line ? parseInt(data.line, 10) : undefined,
        from: data.from
    };
}
/**
 * Converts the output produced by the MI Output parser from the response to the
 * -var-list-children MI command into an array of objects that conform to the IWatchChildInfo
 * interface.
 */
function extractWatchChildren(data) {
    var extractWatchChild = function (data) {
        return {
            id: data.name,
            childCount: parseInt(data.numchild),
            value: data.value,
            expressionType: data['type'],
            threadId: parseInt(data['thread-id']),
            hasMoreChildren: data.has_more !== '0',
            isDynamic: data.dynamic === '1',
            displayHint: data.displayhint,
            expression: data.exp,
            isFrozen: data.frozen === '1'
        };
    };
    if ((data === undefined) || Array.isArray(data)) {
        // data will only be an array if the array is empty
        return [];
    }
    else if (Array.isArray(data.child)) {
        // data is in the form: { child: [{ name: var1.child1, ... }, { name: var1.child2, ... }, ...]
        return data.child.map(function (child) { return extractWatchChild(child); });
    }
    else {
        // data is in the form: { child: { name: var1.child1, ... } }
        return [extractWatchChild(data.child)];
    }
}
/**
 * Converts the output produced by the MI Output parser from the response to the
 * -data-disassemble MI command into an array of objects that conform to the IAsmInstruction
 * interface.
 */
function extractAsmInstructions(data) {
    return data.map(function (asmInstruction) {
        return {
            address: asmInstruction.address,
            func: asmInstruction['func-name'],
            offset: parseInt(asmInstruction.offset),
            inst: asmInstruction.inst,
            opcodes: asmInstruction.opcodes,
            size: parseInt(asmInstruction.size, 10)
        };
    });
}
/**
 * Converts the output produced by the MI Output parser from the response to the
 * -data-disassemble MI command into an array of objects that conform to the ISourceLineAsm
 * interface.
 */
function extractAsmBySourceLine(data) {
    var extractSrcAsmLine = function (data) {
        return {
            line: parseInt(data.line, 10),
            file: data.file,
            fullname: data.fullname,
            instructions: extractAsmInstructions(data.line_asm_insn)
        };
    };
    if ((data === undefined) || Array.isArray(data)) {
        // data will only be an array if the array is empty
        return [];
    }
    else if (Array.isArray(data.src_and_asm_line)) {
        // data is in the form:  { src_and_asm_line: [{ line: "45", ... }, { line: "46", ... }, ...] }
        return data.src_and_asm_line.map(extractSrcAsmLine);
    }
    else {
        // data is in the form: { src_and_asm_line: { line: "45", ... } }
        return [extractSrcAsmLine(data.src_and_asm_line)];
    }
}
function setProcessEnvironment() {
    // HACK for LLDB on Windows (where users have to build their own Python)
    if (os.platform() === 'win32') {
        if (process.env['LLDB_PYTHON_SRC'] === undefined) {
            throw new Error('LLDB_PYTHON_SRC environment variable is not set. It must be set to the source directory ' +
                'of the Python version used in the LLDB build.');
        }
        if (process.env['LLVM_SRC_BUILD'] === undefined) {
            throw new Error('LLVM_SRC_BUILD environment variable is not set. It must be set to the LLVM build output ' +
                'directory.');
        }
        process.env['PATH'] =
            process.env['PATH'] + ';' + path.join(process.env['LLDB_PYTHON_SRC'], 'PCbuild');
        var pythonPath = path.join(process.env['LLDB_PYTHON_SRC'], 'Lib') + ';' +
            path.join(process.env['LLVM_SRC_BUILD'], 'lib\\site-packages');
        if (process.env['PYTHONPATH']) {
            process.env['PYTHONPATH'] = process.env['PYTHONPATH'] + ';' + pythonPath;
        }
        else {
            process.env['PYTHONPATH'] = pythonPath;
        }
    }
}
/**
 * Uses a pseudo-terminal to forward target stdout when doing local debugging.
 *
 * GDB only forwards stdout from the target via async notifications when remote debugging,
 * when doing local debugging it expects the front-end to read the target stdout via a
 * pseudo-terminal. This distinction between remote/local debugging seems annoying, so when
 * debugging a local target this class automatically creates a pseudo-terminal, reads the target
 * stdout, and emits the text via [[EVENT_TARGET_OUTPUT]]. In this way the front-end using this
 * library doesn't have to bother creating pseudo-terminals when debugging local targets.
 */
var GDBDebugSession = (function (_super) {
    __extends(GDBDebugSession, _super);
    function GDBDebugSession() {
        _super.apply(this, arguments);
        /** `true` if this is a remote debugging session. */
        this.isRemote = false;
    }
    GDBDebugSession.prototype.end = function (notifyDebugger) {
        var _this = this;
        if (notifyDebugger === void 0) { notifyDebugger = true; }
        return new Promise(function (resolve, reject) {
            if (_this.terminal) {
                _this.terminal.destroy();
                _this.terminal = null;
            }
            resolve();
        })
            .then(function () { return _super.prototype.end.call(_this, notifyDebugger); });
    };
    GDBDebugSession.prototype.canEmitFunctionFinishedNotification = function () {
        return true;
    };
    GDBDebugSession.prototype.connectToRemoteTarget = function (host, port) {
        var _this = this;
        return _super.prototype.connectToRemoteTarget.call(this, host, port)
            .then(function () { _this.isRemote = true; });
    };
    GDBDebugSession.prototype.startInferior = function (options) {
        var _this = this;
        if (this.isRemote) {
            return _super.prototype.startInferior.call(this, options);
        }
        else {
            return new Promise(function (resolve, reject) {
                if (_this.terminal) {
                    _this.terminal.destroy();
                    _this.terminal = null;
                }
                _this.terminal = pty.open();
                _this.terminal.on('data', function (data) {
                    _this.emit(DebugSession.EVENT_TARGET_OUTPUT, data);
                });
                resolve();
            })
                .then(function () { return _this.setInferiorTerminal(_this.terminal.pty); })
                .then(function () { return _super.prototype.startInferior.call(_this, options); });
        }
    };
    return GDBDebugSession;
})(DebugSession);
/**
 * Starts a new debugging session and spawns the debbuger process.
 *
 * Once the debug session has outlived its usefulness call [[DebugSession.end]] to ensure proper
 * cleanup.
 *
 * @returns A new debug session, or null if a new session couldn't be started.
 */
function startDebugSession(debuggerName) {
    var debuggerFilename;
    var debuggerArgs;
    switch (debuggerName) {
        case 'lldb':
        case 'lldb-mi':
            setProcessEnvironment();
            // lldb-mi.exe should be on the PATH
            debuggerFilename = 'lldb-mi';
            debuggerArgs = ['--interpreter'];
            break;
        case 'gdb':
            debuggerFilename = 'gdb';
            debuggerArgs = ['--interpreter', 'mi'];
            break;
        default:
            throw new Error('Unknown debugger: ' + debuggerName);
            break;
    }
    var debuggerProcess = child_process_1.spawn(debuggerFilename, debuggerArgs);
    var debugSession = null;
    if (debuggerProcess) {
        if (debuggerName === 'gdb') {
            debugSession = new GDBDebugSession(debuggerProcess.stdout, debuggerProcess.stdin);
        }
        else {
            debugSession = new DebugSession(debuggerProcess.stdout, debuggerProcess.stdin);
        }
        if (debugSession) {
            debuggerProcess.once('exit', function (code, signal) { debugSession.end(false); });
        }
    }
    return debugSession;
}
exports.startDebugSession = startDebugSession;
;
// There are more reasons listed in the GDB/MI spec., the ones here are just the subset that's 
// actually used by LLDB MI at this time (11-Apr-2015).
var targetStopReasonMap = new Map()
    .set('breakpoint-hit', TargetStopReason.BreakpointHit)
    .set('end-stepping-range', TargetStopReason.EndSteppingRange)
    .set('function-finished', TargetStopReason.FunctionFinished)
    .set('exited-normally', TargetStopReason.ExitedNormally)
    .set('signal-received', TargetStopReason.SignalReceived)
    .set('exception-received', TargetStopReason.ExceptionReceived);
function parseTargetStopReason(reasonString) {
    var reasonCode = targetStopReasonMap.get(reasonString);
    if (reasonCode !== undefined) {
        return reasonCode;
    }
    // TODO: log and keep on running
    return TargetStopReason.Unrecognized;
}
/**
 * Parses a list of stopped threads from a GDB/MI 'stopped' async notification.
 * @return An array of thread identifiers, an empty array is used to indicate that all threads
 *         were stopped.
 */
function parseStoppedThreadsList(stoppedThreads) {
    if (stoppedThreads === 'all') {
        return [];
    }
    else {
        // FIXME: GDB/MI spec. fails to specify what the format of the list is, need to experiment
        //        to figure out what is actually produced by the debugger.
        return [parseInt(stoppedThreads, 10)];
    }
}
// maps WatchFormatSpec enum members to the corresponding MI string
var watchFormatSpecToStringMap = new Map()
    .set(WatchFormatSpec.Binary, 'binary')
    .set(WatchFormatSpec.Decimal, 'decimal')
    .set(WatchFormatSpec.Hexadecimal, 'hexadecimal')
    .set(WatchFormatSpec.Octal, 'octal')
    .set(WatchFormatSpec.Default, 'natural');
var stringToWatchAttributeMap = new Map()
    .set('editable', WatchAttribute.Editable)
    .set('noneditable', WatchAttribute.NonEditable);
// maps RegisterValueFormatSpec enum members to the corresponding MI code
var registerValueFormatSpecToCodeMap = new Map()
    .set(RegisterValueFormatSpec.Binary, 't')
    .set(RegisterValueFormatSpec.Decimal, 'd')
    .set(RegisterValueFormatSpec.Hexadecimal, 'x')
    .set(RegisterValueFormatSpec.Octal, 'o')
    .set(RegisterValueFormatSpec.Raw, 'r')
    .set(RegisterValueFormatSpec.Default, 'N');
//# sourceMappingURL=dbgmits.js.map